/* $Id: e2p_for_each.c 1890 2006-05-28 02:47:22Z tpgww $
 *
 * Copyright (C) 2003-2006 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_filelist.h"

/**
 * @brief establish and show the dialog
 * This action may take its command from runtime data, or if the latter is
 * NULL, will ask for the command here
 * @param from the button, menu item etc which was activated
 * @param art action runtime data
 *
 * @return TRUE if action completed successfully, else FALSE
 */
static gboolean _e2p_foreach (gpointer from, E2_ActionRuntime *art)
{
  gchar *s, *utf;
  GString *command;
  GList *base, *tmp;
  FileInfo *info;
  DialogButtons result;
  gint res;

  if (art->data == NULL)
  {
	result = e2_dialog_line_input (_("repeat action"),
		_("Enter an action to execute on each\n"
		"selected file:"),
		"", 0, FALSE, &s);
  }
  else
  {
    s = (gchar *)art->data;
	result = OK;
  }

  if (result == OK)
  { //a blank entry will not return OK
   	e2_filelist_disable_refresh ();
	//FIXME Q this
  	base = tmp = e2_fileview_get_selected_local (curr_view);
    if (base == NULL)
	{
		if (art->data == NULL)
			g_free (s);
	   	e2_filelist_enable_refresh ();
		return FALSE;
	}
	//FIXME support macros other than with english letters
	//FIXME support cancellation by user eg via %{}
    if (strstr (s, "%f") == NULL)
    {
      command = g_string_sized_new(128);
      for (; tmp != NULL; tmp = tmp->next)
      {
        info = tmp->data;
		utf = F_FILENAME_FROM_LOCALE (info->filename);
        g_string_printf (command, "%s \"%s\"", s, utf);
		res = e2_command_run (command->str, E2_COMMAND_RANGE_DEFAULT);
		F_FREE (utf);
		if (res != 0)
			break;	//command did not complete properly
      }
      g_string_free (command, TRUE);
    }
    else	//the command included "%f" already
    {
      for (; tmp != NULL; tmp = tmp->next)
      {
        info = tmp->data;
		utf = F_FILENAME_FROM_LOCALE (info->filename);
		command = e2_utils_expand_macros (s, utf);
		if (command == NULL	//error
		 || command == GINT_TO_POINTER (1))//user aborted
		{
			F_FREE (utf);
			break;
		}
        res = e2_command_run (command->str, E2_COMMAND_RANGE_DEFAULT);
        g_string_free (command, TRUE);
		F_FREE (utf);
		if (res != 0)
			break;
      }
    }
	if (art->data == NULL)
	  g_free (s);
    g_list_free (base);
   	e2_filelist_enable_refresh ();
	return TRUE;
  }
  return FALSE;
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "foreach"
  aname = _("foreach");

  p->signature = ANAME VERSION;
  p->menu_name = _("For _each..");
  p->description = _("Execute an entered command on each selected item separately");
  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate

  if (p->action == NULL)
  {
	//no need to free this
	gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
	p->action = e2_plugins_action_register
	  (action_name, E2_ACTION_TYPE_ITEM, _e2p_foreach, NULL, TRUE, 0, NULL);
	return TRUE;
  }
  return FALSE;
}
/**
 * @brief cleanup transient things for this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return TRUE if all cleanups were completed
 **/
gboolean clean_plugin (Plugin *p)
{
  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
  gboolean ret = e2_plugins_action_unregister (action_name);
  g_free (action_name);
  return ret;
}
