/* $Id: e2p_pack.c 2172 2006-07-28 03:30:57Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_command.h"
#ifdef E2_ASYNC
#include "e2_task.h"
#endif
#include "e2_filelist.h"

static GtkWidget *filename_entry;
static GtkWidget *pkgtype_combo;

enum { TAR_GZ, TAR_BZ2, TAR, ZIP, RAR, ARJ, MAXTYPES };

#ifdef E2_ASYNC
static gboolean _e2p_packQ (E2_ActionTaskData *qed);
#endif

static gboolean _e2p_pack_no_cb (GtkWidget *widget, GtkWidget *dialog)
{
//  gtk_widget_set_sensitive (app.main_window, TRUE);
  gtk_widget_destroy (dialog);
#ifndef E2_ASYNC
  gtk_main_quit ();
#endif
  return TRUE;
}

static gboolean _e2p_pack_yes_cb (GtkWidget *widget, GtkWidget *dialog)
{
  static gchar *name_str [MAXTYPES] =
  {	//these extension strings are in same order as enum
    ".tar.gz",
    ".tar.bz2",
    ".tar",
    ".zip",
	".rar",
	".arj"
  };
  static gchar *cmd_str [MAXTYPES] =
  {	//these command strings are in same order as enum
	  //NOTE that %%f is converted to %f when the command is constructed with g_strdup_printf ()
    ">tar cvf - %%f | gzip - > \"%s\"",	//run in separate shell
    ">tar cvf - %%f | bzip2 - > \"%s\"",	//ditto
    "tar cvf \"%s\" %%f",
    "zip -r \"%s\" %%f",
	"rar a -r -ol \"%s\" %%f",
	"arj a -a1 -r \"%s\" %%f"
  };

  gboolean flag = FALSE;
  gint pkg_type = gtk_combo_box_get_active (GTK_COMBO_BOX (pkgtype_combo));
  if (pkg_type != -1)
  {
	gchar *full_name;
	const gchar *chosen_name = gtk_entry_get_text (GTK_ENTRY (filename_entry));
	flag = (*chosen_name != '\0');
	if (flag)
	{
		full_name = g_strconcat (chosen_name, name_str [pkg_type], NULL);
#ifdef E2_ASYNC
		_e2p_pack_no_cb (widget, dialog);	//kill the dialog asap
#endif
		if (e2_option_bool_get ("confirm-overwrite"))
		{
#ifdef E2_ASYNC
			//FIXME qed->currdir is needed instead of curr_view->dir
			gchar *utf = g_strconcat (curr_view->dir, full_name, NULL);  //separator comes with dir
			gchar *dlocal = F_FILENAME_TO_LOCALE (utf);
#else
			gchar *utf = g_strconcat (curr_view->dir, full_name, NULL);  //separator comes with dir
			gchar *dlocal = F_FILENAME_TO_LOCALE (utf);
#endif
			if (e2_fs_access2 (dlocal) == 0)
			{	//same-named item exists already
				DialogButtons result = e2_dialog_ow_check (NULL, dlocal, NONE);
				if (result != OK)
				{
					flag = FALSE;	//signal that we will not proceed
					g_free (full_name);
				}
			}
#ifdef E2_ASYNC
			//FIXME
			g_free (utf);
			F_FREE (dlocal);
#else
			g_free (utf);
			F_FREE (dlocal);
#endif
		}
	}
#ifdef E2_ASYNC
	else
		_e2p_pack_no_cb (widget, dialog);
#endif
	if (flag)
	{
		//FIXME find a way to make the lists refresh when the packing is done
		//appending ";panes.refresh" to the command doesn't work
		gchar *command =  g_strdup_printf (cmd_str [pkg_type], full_name);
		e2_filelist_disable_refresh ();	//prevent changes to selected item data
		gint res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
		e2_filelist_enable_refresh (); //probably nothing reported yet, to trigger a refresh
		flag = (res == 0);
    	g_free (command);
		g_free (full_name);
	}
  }
#ifdef E2_ASYNC
  else	//some invalid package type was selected
	_e2p_pack_no_cb (widget, dialog);
#else
  _e2p_pack_no_cb (widget, dialog);
#endif

  return flag;
}

static gboolean _e2p_pack_delete_cb (GtkWidget *widget,
	GdkEvent *event, GtkWidget *dialog)
{
    return _e2p_pack_no_cb (widget, dialog);
}

static gboolean _e2p_pack_key_press_cb (GtkWidget *widget,
	GdkEventKey *event, GtkWidget *dialog)
{
  if (event->keyval == GDK_Return)
	  return _e2p_pack_yes_cb (widget, dialog);
  else if (event->keyval == GDK_Escape)
    return _e2p_pack_no_cb (widget, dialog);

  return FALSE;
}

/**
 * @brief create and run dialog
 *
 * @param from the button, menu item etc which was activated
 * @param art action runtime data
 *
 * @return TRUE if action completed successfully, else FALSE
 */
static gboolean _e2p_pack (gpointer from, E2_ActionRuntime *art)
{
#ifdef E2_ASYNC
	return (e2_task_enqueue_task (E2_TASK_PACK, art, from,
		_e2p_packQ, NULL));
}
static gboolean _e2p_packQ (E2_ActionTaskData *qed)
{
	GPtrArray *names = qed->names;
#else
	//handle case of specific data instead of selection??
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL)
		return FALSE;	//nothing selected
#endif

  GtkWidget *dialog = e2_dialog_create (NULL,
	_("Filename:"), _("archive creation"), NULL, NULL);

  GtkWidget *hbox = g_object_get_data (G_OBJECT (dialog), "e2-dialog-hbox");

  E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
  gchar *suggested_name = F_FILENAME_FROM_LOCALE ((*iterator)->filename);
  filename_entry = e2_widget_add_entry (hbox, suggested_name, TRUE, TRUE);
  F_FREE (suggested_name);

  pkgtype_combo = gtk_combo_box_entry_new_text ();
  //these package name strings are in same order as enum
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar.gz"));  //index 0
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar.bz2"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".tar"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".zip"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".rar"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (pkgtype_combo), _(".arj"));  //index 5
  gtk_combo_box_set_active (GTK_COMBO_BOX (pkgtype_combo), 0);
  gtk_widget_set_size_request (pkgtype_combo, 120, -1);  //'natural' width (= filename entry width) too big
  gtk_box_pack_start (GTK_BOX (hbox), pkgtype_combo, TRUE, TRUE, 0);
  gtk_widget_show (pkgtype_combo);

  g_signal_connect (G_OBJECT (dialog), "delete_event",
	G_CALLBACK (_e2p_pack_delete_cb), dialog);
  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2p_pack_key_press_cb), dialog);

  e2_dialog_add_button_custom (dialog, FALSE, &E2_BUTTON_CANCEL,
	NULL, _e2p_pack_no_cb, dialog);
  e2_dialog_add_button_custom (dialog, TRUE, &E2_BUTTON_OK,
	NULL, _e2p_pack_yes_cb, dialog);

#ifdef E2_ASYNC
  gdk_threads_enter ();
#endif
  e2_dialog_setup (dialog, app.main_window);
  gtk_widget_show (dialog);
//  gtk_widget_set_sensitive (app.main_window, FALSE);
#ifdef E2_ASYNC
  gdk_threads_leave ();
#endif
#ifndef E2_ASYNC
  gtk_main();
#endif
  return TRUE;
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "pack"
  aname = _("pack");

  p->signature = ANAME VERSION;
  p->menu_name = _("_Pack..");
  p->description = _("Build an archive containing the selected item(s)");
  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate

  if (p->action == NULL)
  {
	//no need to free this
	gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
	p->action = e2_plugins_action_register
	  (action_name, E2_ACTION_TYPE_ITEM, _e2p_pack, NULL, FALSE, 0, NULL);
	return TRUE;
  }
  return FALSE;
}
/**
 * @brief cleanup transient things for this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return TRUE if all cleanups were completed
 **/
gboolean clean_plugin (Plugin *p)
{
  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
  gboolean ret = e2_plugins_action_unregister (action_name);
  g_free (action_name);
  return ret;
}
