/* $Id: e2_edit_dialog.c 2187 2006-07-31 02:00:36Z tpgww $
 *
 * Copyright (C) 2006 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_dialog.h"
#include "e2_view_dialog.h"
#include "e2_textiter.h"
#include "e2_task.h"

//rates around the default blink rate will never actually blink! dunno why ...
//so: MULTIPLIER >= 1.5, MULTIPLIER/DIVIDER <= 0.5
#define OVERWR_CURSOR_MULTIPLIER 2.0
#define INSERT_CURSOR_DIVIDER 4.0
#ifndef CURSOR_ON_MULTIPLIER
#define CURSOR_ON_MULTIPLIER 0.66
#endif
#ifndef CURSOR_OFF_MULTIPLIER
#define CURSOR_OFF_MULTIPLIER 0.34
#endif

//these variables can all be shared among all concurrent and sequential
//view and edit dialogs
extern E2_OptionSet *opt_view_width;
extern E2_OptionSet *opt_view_height;
extern E2_OptionSet *opt_view_wrap;
extern E2_OptionSet *opt_show_last;
extern E2_OptionSet *opt_case_sensitive;
extern gboolean show_last;
extern gboolean case_sensitive;
extern gboolean search_backward;
extern gboolean search_wrap;
extern GList *find_history;
gboolean replace_all;
gboolean confirm_before;
gboolean do_next;
static GList *replace_history = NULL;

gint blinkmsec;	//blink-cycle duration (msec)
//keycodes corresponding to mnemonics for translated labels
//these ones are shared with a view dialog
extern guint find_keycode;
extern guint hide_keycode;
guint save_keycode;
guint replace_keycode;
guint undo_keycode;
guint redo_keycode;

static void _e2_edit_dialog_dirty_change_cb (GtkTextBuffer *textbuffer,
	E2_ViewDialogRuntime *rt);
#ifdef E2_ASYNC
static gboolean _e2_edit_dialog_reeditQ (E2_ActionTaskData *qed);
#endif

  /****************************/
 /**** undo functionality ****/
/****************************/

#define INCLUDED_IN_PARENT
#include "e2_edit_dialog_undo.c"
#undef INCLUDED_IN_PARENT

  /***************************/
 /**** editing functions ****/
/***************************/

/**
 * @brief save text selected in @a buffer
 * This func is also used to save output pane selection
 * @param buffer text buffer from which the text will be saved
 * @param parent parent widget
 *
 * @return
 **/
void e2_edit_dialog_save_selected (GtkTextBuffer *buffer, GtkWidget *parent)
{
	GtkTextIter start, end;
	//get this first, as the dialog unselects the text
	if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end))
	{
		//FIXME vfs
		GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
			GTK_WINDOW (parent),
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);
		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
		E2_DIALOG_TITLE (GTK_WINDOW (dialog), _("file save as"));
		gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), FALSE);
		gboolean check = e2_option_bool_get ("confirm-overwrite");
#if defined (E2_CURRENTGTK_VERSION) && GTK_CHECK_VERSION (2, 8, 0)
		if (check)
			gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
#endif
		//the API doco is silent on whether g_get_home_dir() gets a localised string
		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), g_get_home_dir ());
		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
		{
			gchar *dlocal = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
			gchar *utf = F_FILENAME_FROM_LOCALE (dlocal);
#if !defined (E2_CURRENTGTK_VERSION) || !GTK_CHECK_VERSION (2, 8, 0)
			//check for o/w
			if (check && e2_fs_access2 (dlocal) == 0)
			{
				DialogButtons choice = e2_dialog_ow_check (NULL, dlocal, NONE);
				if (choice != OK)
				{
					gtk_widget_destroy (dialog);
					gtk_text_buffer_select_range (buffer, &start, &end);
					g_free (dlocal);
					F_FREE (utf);
					return;
				}
			}
#endif
			E2_FILE *file = e2_fs_open_file_to_write (utf);
			if (file != NULL)
			{
				gchar *text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
				e2_fs_file_put (utf, text, file);
				e2_fs_file_close (file);
				g_free (text);
			}
			g_free (dlocal);
			F_FREE (utf);
		}
		gtk_widget_destroy (dialog);
		gtk_text_buffer_select_range (buffer, &start, &end);
	}
}
/**
 * @brief save currently-selected text
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_savesel_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	e2_edit_dialog_save_selected (rt->textbuffer, rt->dialog);
}
/**
 * @brief save file
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_save_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	rt->saved_ok = FALSE;
	GtkTextIter start, end;
	gtk_text_buffer_get_bounds (rt->textbuffer, &start, &end);
	gchar *text = gtk_text_buffer_get_text (rt->textbuffer, &start, &end, FALSE);

	//restore line ends
	gint extras;
	if (rt->linebreak == CR+LF)
		extras = gtk_text_buffer_get_line_count (rt->textbuffer) + 1;	//+1 = possible incomplete line
	else
		extras = 0; //irrelevant
	text = e2_utils_revert_line_ends (text, extras, rt->linebreak);

	//after external encoding conversion, or some fallback conversion,
	//we can't know the original encoding, so just save as UTF-8, and warn the user
	if (!g_str_equal (rt->charset, "UNKNOWN")
		&& strstr (rt->charset, "UTF-8") == NULL)	//conversion is actually needed
	{
		gsize bytes_old, bytes_new;
		GError *error = NULL;
		text = g_convert (text, -1, rt->charset, "UTF-8", &bytes_old, &bytes_new, &error);
		if (error != NULL)
		{
			gchar *msg;
			switch (error->code)
			{
				case G_CONVERT_ERROR_ILLEGAL_SEQUENCE:
					msg = g_strdup_printf (
					_("Cannot save %s in its original encoding %s. Reverting to UTF-8"),
					rt->filepath, rt->charset);
					e2_output_print_error (msg, TRUE);
					g_error_free (error);
					break;
				default:
					msg = g_strdup_printf (
					_("Encoding conversion failed for %s"), rt->filepath);
					e2_output_print_error (msg, TRUE);
					e2_output_print_error (error->message, FALSE);
					g_error_free (error);
					return;
					break;
			}
		}
	}

	gchar *local = F_FILENAME_TO_LOCALE (rt->filepath);
	struct stat sb;
	gboolean renamed = FALSE;
	if (e2_option_bool_get ("edit-save-backup")
		&& !e2_fs_access (local, W_OK))
	{
		renamed = !e2_fs_stat (local, &sb);
		gchar *saved = e2_utils_strcat (local, "~");
		e2_task_backend_rename (local, saved);	//ignore any warning !!
		g_free (saved);
	}

	E2_FILE *file = e2_fs_open_file_to_write (rt->filepath);
	if (file == NULL)
	{
		g_free (text);
		F_FREE (local);
		return;
		//FIXME undo backup-rename
	}
	if (e2_fs_file_put (rt->filepath, text, file) == EOF)
	{
		g_free (text);
		F_FREE (local);
		e2_fs_file_close (file);
		return;
		//FIXME undo backup-rename
	}
	e2_fs_file_close (file);

	if (renamed)
	{
		//make new file the same as the old one
		e2_fs_chmod (local, sb.st_mode);
//		uid_t owner_id = ?;
//		gid_t group_id = ?;
//		e2_fs_chown (local, owner_id, group_id);
	}

	g_free (text);
	F_FREE (local);

	if (g_str_equal (rt->charset, "UNKNOWN"))
	{
		gchar *msg = g_strdup_printf
		(_("Original encoding of '%s' is unknown, now saved as UTF-8"), rt->filepath);
		e2_output_print_error (msg, TRUE);
	}
	rt->saved_ok = TRUE;
	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (rt->textbuffer), FALSE);
}
/**
 * @brief save file with new name
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_saveas_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{	//FIXME vfs
	//CHECKME save on <Enter> press ?
	GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL,
		GTK_WINDOW (rt->dialog),
		GTK_FILE_CHOOSER_ACTION_SAVE,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_SAVE, GTK_RESPONSE_OK,
		NULL);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
	E2_DIALOG_TITLE (GTK_WINDOW (dialog), _("file save as"));
	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), FALSE);
	gboolean check = e2_option_bool_get ("confirm-overwrite");
#if defined (E2_CURRENTGTK_VERSION) && GTK_CHECK_VERSION (2, 8, 0)
	if (check)
		gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
#endif
	gchar *local = F_FILENAME_TO_LOCALE (rt->filepath);
//this does not work as stated ..
//	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), local);
	gchar *name = g_path_get_basename (local);
	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), name);
	g_free (name);
	name = g_path_get_dirname (local);
	if (!g_str_equal (name, "."))
		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), name);
	g_free (name);
	F_FREE (local);

	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
	{
		gchar *dlocal = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
#if !defined (E2_CURRENTGTK_VERSION) || !GTK_CHECK_VERSION (2, 8, 0)
		//check for o/w
		if (check && e2_fs_access2 (dlocal) == 0)
		{
			gchar *utf = F_FILENAME_FROM_LOCALE (dlocal);
			DialogButtons choice = e2_dialog_ow_check (NULL, dlocal, NONE);
			if (choice != OK)
			{
				gtk_widget_destroy (dialog);
				g_free (dlocal);
				F_FREE (utf);
				return;
			}
			g_free (rt->filepath);
			rt->filepath = utf;
		}
		else
		{
#endif
			g_free (rt->filepath);
			rt->filepath = F_FILENAME_FROM_LOCALE (dlocal);
#if !defined (E2_CURRENTGTK_VERSION) || !GTK_CHECK_VERSION (2, 8, 0)
		}
#endif
		if (rt->filepath != dlocal)
			g_free (dlocal);
		_e2_edit_dialog_save_cb (NULL, rt);

		if (rt->saved_ok)
		{	//update the dialog label
			GtkWidget *label = g_object_get_data (G_OBJECT (rt->dialog), "e2-dialog-label");
			gtk_label_set_text (GTK_LABEL (label), rt->filepath);
		}
	}
	gtk_widget_destroy (dialog);
}
/**
 * @brief reload edited file
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_refresh_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	e2_view_dialog_read_text (rt);	//probably won't fail when refreshing
	gtk_text_buffer_set_modified (rt->textbuffer, FALSE);
	gtk_widget_grab_focus (rt->textview);
}
/**
 * @brief undo change
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_undo_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	_e2edtdlg_undo_undo_one (rt);
}
#ifdef E2_REDO_ENABLED
/**
 * @brief redo change
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_redo_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	_e2edtdlg_undo_redo_one (rt);
}
#endif
/**
 * @brief toggle text wrapping
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_wrap_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	rt->textwrap = !rt->textwrap;
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
		rt->textwrap ? GTK_WRAP_WORD: GTK_WRAP_NONE);
}
/**
 * @brief hide search/replace bar
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_hide_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	gtk_widget_hide (rt->panel);
	rt->is_hidden = TRUE;
	gtk_widget_hide (rt->info_label);
	gtk_widget_hide (rt->findbtn);
	gtk_widget_hide (rt->replacebtn);
	//in case we last performed an incremental search ...
	GtkWidget *entry = GTK_BIN (rt->combo)->child;
	const gchar *choice = gtk_entry_get_text (GTK_ENTRY (entry));
	if (*choice != '\0')
	{
		e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
		e2_list_update_history (choice, &find_history, NULL, 30, FALSE);
	}
	//now for the replacements ...
	entry = GTK_BIN (rt->combo2)->child;
	choice = gtk_entry_get_text (GTK_ENTRY (entry));
	if (*choice != '\0')
	{
		e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
		e2_list_update_history (choice, &replace_history, NULL, 30, FALSE);
	}
	//focus on the text
	gtk_widget_grab_focus (rt->textview);
}
/**
 * @brief perform find
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_find_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	if (GTK_WIDGET_VISIBLE (rt->panel)) //search panel is visible
	{
		//if a mod key is pressed, treat it specially
		//Ctrl = find first, Shift = temp reverse direction
		GdkModifierType mask = e2_utils_get_modifiers ();
		if (mask & GDK_SHIFT_MASK)
		{	//temporary direction change
			search_backward = !search_backward;
			e2_view_dialog_search (FALSE, FALSE, rt);
			search_backward = !search_backward;
		}
		else if (mask & GDK_CONTROL_MASK)
			e2_view_dialog_search (TRUE, FALSE, rt);
		else
			e2_view_dialog_search (FALSE, FALSE, rt);
	}
	else
	{	//show the search panel & related things
		if (GTK_WIDGET_VISIBLE (rt->replacebar))
			gtk_widget_hide (rt->replacebar);
		gtk_widget_show (rt->panel);
		rt->is_hidden = FALSE;
		gtk_widget_show (rt->findbtn);
		//focus on the text entry
		gtk_widget_grab_focus (GTK_BIN (rt->combo)->child);
	}
}
/**
 * @brief perform replacement
 *
 * @param menuitem UNUSED the selected widget, or NULL
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_replace_cb (GtkWidget *menuitem,
	E2_ViewDialogRuntime *rt)
{
	if (!GTK_WIDGET_VISIBLE (rt->replacebtn))
		gtk_widget_show (rt->replacebtn);
	if (!GTK_WIDGET_VISIBLE (rt->findbtn))
		gtk_widget_show (rt->findbtn);
	//search bar is always visible, but ...
	if (!GTK_WIDGET_VISIBLE (rt->replacebar))
		gtk_widget_show (rt->replacebar);

	if (GTK_WIDGET_VISIBLE (rt->panel)) //search panel is visible
	{
		GtkTextIter start, end;
		if (!gtk_text_buffer_get_selection_bounds (rt->textbuffer, &start, &end))
		{	//nothing selected, presumably after a replace that's just happened
			//lazy user, try for another match, like Ctrl F
			e2_view_dialog_search (FALSE, FALSE, rt);
			return;
		}
		GtkWidget *entry = GTK_BIN (rt->combo2)->child;
		const gchar *replace = gtk_entry_get_text (GTK_ENTRY (entry));
		//do replacement(s)
		if (replace_all)
		{
			GtkWidget *dialog;//ignore compiler warning about unitialized usage
			DialogButtons choice;
			if (confirm_before)
			{
				dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
				_("Replace this one ?"), _("confirm"), e2_dialog_response_decode_cb, &choice);
				e2_dialog_add_defined_button (dialog, &E2_BUTTON_NOTOALL);
				e2_dialog_add_defined_button (dialog, &E2_BUTTON_NO);
				e2_dialog_add_defined_button (dialog, &E2_BUTTON_YES);
				e2_dialog_setup (dialog, rt->dialog);
			}
			while (e2_view_dialog_search (FALSE, FALSE, rt))
			{
				if (confirm_before)
				{
					gtk_widget_show (dialog);
					gtk_main ();
					gtk_widget_hide (dialog);
					if (choice == CANCEL)
						continue;
					else if (choice == NO_TO_ALL)
						break;
				}
				gtk_text_buffer_delete_selection (rt->textbuffer, TRUE, TRUE);
				if (*replace != '\0')
					gtk_text_buffer_insert_interactive_at_cursor (rt->textbuffer,
						replace, -1, TRUE);
			}
			if (confirm_before)
				gtk_widget_destroy (dialog);
		}
		else if (gtk_text_buffer_get_selection_bounds (rt->textbuffer, &start, &end))
		{
			gtk_text_buffer_delete_selection (rt->textbuffer, TRUE, TRUE);
			if (*replace != '\0')
				gtk_text_buffer_insert_interactive_at_cursor (rt->textbuffer,
					replace, -1, TRUE);
			if (do_next)
				e2_view_dialog_search (FALSE, FALSE, rt);
		}
	}
	else	//first time, make the search panel visible
	{
		gtk_widget_show (rt->panel);
		rt->is_hidden = FALSE;
		gtk_widget_grab_focus (GTK_BIN (rt->combo2)->child);
	}
}
/**
 * @brief construct and show output-pane context menu
 *
 * @param textview the textview widget where the click happened
 * @param event_button which mouse button was clicked (0 for a menu key)
 * @param event_time time that the event happened (0 for a menu key)
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_edit_dialog_show_context_menu (GtkWidget *textview,
	guint event_button, gint event_time, E2_ViewDialogRuntime *rt)
{
	gchar *item_name;
	GtkWidget *menu = gtk_menu_new ();

	GtkWidget *item =
	e2_menu_add (menu, _("_Save"), GTK_STOCK_SAVE,
		_("Save the file"), _e2_edit_dialog_save_cb, rt);
	gtk_widget_set_sensitive (item, rt->is_dirty);
	e2_menu_add (menu, _("Save as.."), GTK_STOCK_SAVE_AS,	//no available valid mnemonic
		_("Save the file with a new name"), _e2_edit_dialog_saveas_cb, rt);
	item =
	e2_menu_add (menu, _("Save se_lection.."), "save_selection_"E2IP".png",
		_("Save the selected text"), _e2_edit_dialog_savesel_cb, rt);
	gtk_widget_set_sensitive (item,
		gtk_text_buffer_get_selection_bounds (rt->textbuffer, NULL, NULL));
	e2_menu_add (menu, _("R_efresh"), GTK_STOCK_REVERT_TO_SAVED,
		_("Reload the file being edited"), _e2_edit_dialog_refresh_cb, rt);
	e2_menu_add (menu, _("_Find.."), GTK_STOCK_FIND,
		_("Find matching text"), _e2_edit_dialog_find_cb, rt);
//	gtk_tooltips_set_tip (app.tooltips, item,
//		_("Show the search options bar"), _("Find the first/next occurrence"));
	e2_menu_add (menu, _("_Replace.."), GTK_STOCK_FIND_AND_REPLACE,
		_("Find and replace matching text"), _e2_edit_dialog_replace_cb, rt);
	item =
	e2_menu_add (menu, _("_Hide"), GTK_STOCK_ZOOM_FIT,
		_("Hide the search options bar"), _e2_edit_dialog_hide_cb, rt);
	gtk_widget_set_sensitive (item, !rt->is_hidden);
	item =
	e2_menu_add (menu, _("_Undo"), GTK_STOCK_UNDO,
		_("Undo last change"), _e2_edit_dialog_undo_cb, rt);
	gtk_widget_set_sensitive (item, rt->undo_enabled);
#ifdef E2_REDO_ENABLED
	item =
	e2_menu_add (menu, _("Re_do"), GTK_STOCK_REDO,
		_("Reverse last undo"), _e2_edit_dialog_redo_cb, rt);
	gtk_widget_set_sensitive (item, rt->redo_enabled);
#endif
	item =
	e2_menu_add_check (menu, _("_Wrap"), rt->textwrap,
		 _e2_edit_dialog_wrap_cb, rt);
	e2_widget_set_tooltip (NULL, item,
		_("If activated, text in the window will be word-wrapped"));

	e2_menu_add_separator (menu);
	item_name = g_strconcat (_A(2),".",_A(32),NULL);
	e2_menu_add_action (menu, _("Se_ttings"), GTK_STOCK_PREFERENCES,
		_("Open the configuration dialog at the options page"),
		item_name, _C(40), //_("view")
		NULL);
	g_free(item_name);

	g_signal_connect (G_OBJECT (menu), "selection-done",
		G_CALLBACK (e2_menu_destroy_cb), NULL);

	if (event_button == 0)
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
			(GtkMenuPositionFunc) e2_view_dialog_set_menu_position,
			rt, event_button, event_time);
	else
		//this was a button-3 click
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
			NULL, NULL, event_button, event_time);
}

  /*****************/
 /***** utils *****/
/*****************/

static void _e2_edit_dialog_destroy (E2_ViewDialogRuntime *rt)
{
#ifdef E2_ASYNC
	g_free ((gchar *)rt->charset);
#endif
	if (rt->blink_id > 0)
//	{
		g_source_remove (rt->blink_id);
//		rt->blink_id = 0;
//	}

	GtkWidget *entry = GTK_BIN (rt->combo2)->child;
	const gchar *repl = gtk_entry_get_text (GTK_ENTRY (entry));
	if (*repl != '\0')
	{
		e2_combobox_activated_cb (entry, GINT_TO_POINTER (FALSE));
		e2_list_update_history (repl, &replace_history, NULL, 30, FALSE);
	}

	GList *tmp;
	for (tmp = app.tabslist; tmp != NULL; tmp = tmp->next)
	{
		if (((E2_OutputTabRuntime*)tmp->data)->buffer == rt->textbuffer)
		{	//we were editing an output pane buffer
			//that stays in existence, so we need to get rid of redundant callbacks
			g_signal_handlers_disconnect_by_func (G_OBJECT (rt->textbuffer),
				_e2edtdlg_insert_text_cb, rt);
			g_signal_handlers_disconnect_by_func (G_OBJECT (rt->textbuffer),
				_e2edtdlg_delete_range_cb, rt);
			g_signal_handlers_disconnect_by_func (G_OBJECT (rt->textbuffer),
				_e2edtdlg_begin_user_action_cb, rt);
			g_signal_handlers_disconnect_by_func (G_OBJECT (rt->textbuffer),
				_e2edtdlg_end_user_action_cb, rt);
			g_signal_handlers_disconnect_by_func (G_OBJECT (rt->textbuffer),
				_e2_edit_dialog_dirty_change_cb, rt);
			break;
		}
	}

	e2_view_dialog_destroy (rt);
}
/**
 * @brief toggle cursor visibility
 * This is a timer callback
 * @param textview the textview for the dialog
 *
 * @return TRUE (so the timer continues) if @a textview is not focused
 **/
// FORCED CURSOR BLINKING STUFFS UP LARGER VERTICAL MOVES
static gboolean _e2_edit_dialog_blink (E2_ViewDialogRuntime *rt)
{
//	printd (DEBUG, "edit dialog cursor visibility toggle cb");
//	gdk_threads_enter ();
//	gboolean blinkme = TRUE;	//GTK_WIDGET_HAS_FOCUS (rt->dialog);
//	if (blinkme)
//	{
//		g_source_remove (blink_id);
	if (GTK_IS_WIDGET (rt->textview))
	{	//the dialog hasn't been closed between callbacks
		gboolean visible = gtk_text_view_get_cursor_visible (GTK_TEXT_VIEW (rt->textview));
		gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (rt->textview), !visible);
		gint period = (visible) ?
			blinkmsec * CURSOR_OFF_MULTIPLIER:
			blinkmsec * CURSOR_ON_MULTIPLIER;
		if (rt->ow_mode)
			period /= INSERT_CURSOR_DIVIDER;
		rt->blink_id = g_timeout_add_full (G_PRIORITY_LOW, period,
			(GSourceFunc) _e2_edit_dialog_blink, rt, NULL);
	}
//	}
//	gdk_threads_leave ();
//	return !blinkme; //remove ourself if we're done
//	return TRUE;	//already removed
	return FALSE;	//remove ourself
}
/**
 * @brief initialize cursor blinking
 * This is an idle callback
 * @param rt data struct for the dialog
 *
 * @return FALSE(so the idle callback is cancelled)
 **/
static gboolean _e2_edit_dialog_blink_start (E2_ViewDialogRuntime *rt)
{
//	printd (DEBUG, "Cursor blinking started");
	if (rt->blink_id == 0)
		rt->blink_id = g_timeout_add_full (G_PRIORITY_HIGH_IDLE,
			blinkmsec * CURSOR_OFF_MULTIPLIER,
			(GSourceFunc) _e2_edit_dialog_blink, rt, NULL);
	rt->blink_init_id = 0;
	return FALSE;
}
/**
 * @brief suspend cursor blinking
 *
 * @param rt data struct for the dialog
 *
 * @return
 **/
static void _e2_edit_dialog_blink_stop (E2_ViewDialogRuntime *rt)
{
	if (rt->blink_id > 0)
	{
		g_source_remove (rt->blink_id);
		rt->blink_id = 0;
	}
	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (rt->textview), TRUE);
	//setup to go again when next idle
	rt->blink_init_id = g_idle_add_full (G_PRIORITY_HIGH_IDLE,
		(GSourceFunc)_e2_edit_dialog_blink_start, rt, NULL);
}

  /*********************/
 /***** callbacks *****/
/*********************/

/**
 * @brief text-buffer edited callback
 *
 * @param textbuffer the textbuffer which is now clean or dirty
 * @param rt dialog runtime data struct
 *
 * @return
 **/
static void _e2_edit_dialog_dirty_change_cb (GtkTextBuffer *textbuffer,
	E2_ViewDialogRuntime *rt)
{
	rt->is_dirty = !rt->is_dirty;
	gtk_widget_set_sensitive (rt->savebtn, rt->is_dirty);
}
/**
 * @brief menu-button press callback
 *
 * @param textview the textview widget where the press happened
 * @param rt dialog runtime data struct
 *
 * @return TRUE always
 **/
static gboolean _e2_edit_dialog_popup_menu_cb (GtkWidget *textview,
	E2_ViewDialogRuntime *rt)
{
	_e2_edit_dialog_blink_stop (rt);
	gint event_time = gtk_get_current_event_time ();
	_e2_edit_dialog_show_context_menu (textview, 0, event_time, rt);
	return TRUE;
}
/**
 * @brief general dialog key press callback
 *
 * @param textview UNUSED the widget where the button was pressed
 * @param event gdk event data
 * @param rt rt data for the dialog
 *
 * @return TRUE (stop other handlers) for recognised keys
**/
gboolean e2_edit_dialog_key_press_cb (GtkWidget *textview,
	GdkEventKey *event, E2_ViewDialogRuntime *rt)
{
	_e2_edit_dialog_blink_stop (rt);
	guint modifiers = gtk_accelerator_get_default_mod_mask ();
	modifiers = event->state & modifiers;
	if (//g_ascii_isalpha (event->keyval) //&& (event->keyval < 0xF000) //&&
		(modifiers == GDK_CONTROL_MASK || modifiers == GDK_MOD1_MASK))
	{	// the key is a letter and the modifier is Control or Alt
		if (event->keyval == save_keycode)
		{
			if (rt->is_dirty)
				_e2_edit_dialog_save_cb (NULL, rt);
			return TRUE;
		}
		else if (event->keyval == find_keycode
				|| event->keyval == GDK_g)	//special case, not in button label
		{
			_e2_edit_dialog_find_cb (NULL, rt);
			return TRUE;
		}
		else if (event->keyval == replace_keycode)
		{
			_e2_edit_dialog_replace_cb (NULL, rt);
			return TRUE;
		}
		else if (event->keyval == hide_keycode)
		{
			_e2_edit_dialog_hide_cb (NULL, rt);
			return TRUE;
		}
		else if (event->keyval == undo_keycode)
		{
			if (rt->undo_enabled)
				_e2edtdlg_undo_undo_one (rt);
			return TRUE;
		}
		else if (event->keyval == GDK_z)
		{
			//no Alt-z support
			if (modifiers == GDK_MOD1_MASK)
				return FALSE;
			if (rt->undo_enabled)
				_e2edtdlg_undo_undo_one (rt);
			return TRUE;
		}
#ifdef E2_REDO_ENABLED
		else if (event->keyval == redo_keycode)
		{
			if (rt->redo_enabled)
				_e2edtdlg_undo_redo_one (rt);
			return TRUE;
		}
#endif
	}
#ifdef E2_REDO_ENABLED
	else if (event->keyval == GDK_z && modifiers == (GDK_CONTROL_MASK | GDK_SHIFT_MASK))
	{
		if (rt->redo_enabled)
			_e2edtdlg_undo_redo_one (rt);
		return TRUE;
	}
#endif
	else if (event->keyval == GDK_Insert)
	{
	//GTK_TEXT_VIEW (rt->textview)->overwrite_mode can get overwrite_mode state
		rt->ow_mode = !gtk_text_view_get_overwrite (GTK_TEXT_VIEW (rt->textview));
		gtk_text_view_set_overwrite (GTK_TEXT_VIEW (rt->textview), rt->ow_mode);
		return TRUE;
	}
//	else if (event->keyval == GDK_Delete)
//	{	//workaround gtk behaviour
//		gtk_text_buffer_set_modified (rt->textbuffer, TRUE);
//	}
	rt->keyval = event->keyval;	//save, for undo processing
	return FALSE;
}
/**
 * @brief replace combo-box key press callback
 * This ensures that a Return keypress behaves like Ctrl-r
 *
 * @param entry UNUSED the widget where the key was pressed
 * @param event gdk event data
 * @param rt rt data for the dialog
 *
 * @return TRUE if Return was pressed
**/
gboolean _e2_edit_dialog_combokey_cb (GtkWidget *entry,
	GdkEventKey *event, E2_ViewDialogRuntime *rt)
{
	switch (event->keyval)
	{
		case GDK_Return:
			_e2_edit_dialog_replace_cb (NULL, rt);
			return TRUE;
		//handle some other special cases too
		case GDK_g:
		case GDK_z:
			return (e2_edit_dialog_key_press_cb (rt->textview, event, rt));
			break;
		default:
			if (event->keyval == find_keycode
			  || event->keyval == replace_keycode
			  || event->keyval == hide_keycode)
			return (e2_edit_dialog_key_press_cb (rt->textview, event, rt));
			break;
	}
	return FALSE;
}
/**
 * @brief mouse button press callback
 *
 * @param textview the widget where the button was pressed
 * @param event gdk event data
 * @param rt rt data for the dialog
 *
 * @return TRUE (stop other handlers) for btn 3 press, else FALSE (allow other handlers)
**/
static gboolean _e2_edit_dialog_button_press_cb (GtkWidget *textview,
	GdkEventButton *event, E2_ViewDialogRuntime *rt)
{
	_e2_edit_dialog_blink_stop (rt);
	if (event->button == 3)
	{
		_e2_edit_dialog_show_context_menu (textview, event->button, event->time, rt);
		return TRUE;
	}
	return FALSE;
}
/**
 * @brief edit dialog response callback
 *
 * @param dialog the dialog widget where the response was triggered
 * @param response the number assigned the activated widget
 * @param view rt data for the dialog
 *
 * @return
**/
static void _e2_edit_dialog_response_cb
	(GtkWidget *dialog, gint response, E2_ViewDialogRuntime *rt)
{
	printd (DEBUG, "response_cb (dialog:_,response:%d,rt:_)", response);
	switch (response)
	{
		case E2_RESPONSE_FIND:
			_e2_edit_dialog_find_cb (NULL, rt);
			break;
		case E2_RESPONSE_USER1:	//replace
			_e2_edit_dialog_replace_cb (NULL, rt);
			break;
		case E2_RESPONSE_USER2:	//save
			_e2_edit_dialog_save_cb (NULL, rt);
			break;
		case GTK_RESPONSE_CLOSE:
			if (rt->is_dirty)
			{
				DialogButtons retval;
				GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
				_("Save modified file ?"), _("confirm"),
				e2_dialog_response_decode_cb, &retval);
				//arrange for a specific retval (NO_TO_ALL) if the user presses Esc
				e2_dialog_set_negative_response (dialog, E2_RESPONSE_USER1);
				e2_dialog_show (dialog, app.main_window,
					0, &E2_BUTTON_NO, &E2_BUTTON_YES, NULL);
				gtk_main ();
				gtk_widget_destroy (dialog);
				if (retval == NO_TO_ALL)
					break;	//Escape in the save dialog just cancels the close, in effect
				if (retval != CANCEL)
					_e2_edit_dialog_save_cb (NULL, rt);
			}
			if (rt->blink_init_id > 0)
			{
				g_source_remove (rt->blink_init_id);
				rt->blink_init_id = 0;
			}
			if (rt->blink_id > 0)
			{
				g_source_remove (rt->blink_id);
				rt->blink_id = 0;
			}
			_e2_edit_dialog_destroy (rt);
			break;
		default:
			break;
	}
}

#ifdef E2_TRANSIENTKEYS
/**
 * @brief function to setup default key-bindings for edit dialog
 *
 * @param set pointer to option data struct
 *
 * @return
**/
static void _e2_edit_dialog_keybindings (E2_OptionSet *set)
{
	//the key name strings are parsed by gtk, and except for simple letters
	//(which might reasonably change for different languages) no translation
	//is possible
	e2_option_tree_setup_defaults (set,
	g_strdup("keybindings=<"),  //internal name
	//the category string(s) here need to match the binding name
//	g_strconcat(_C(16),"||||",NULL),  //_("general"
	g_strconcat(_C(9),"||||",NULL),  //_("dialogs"
	g_strconcat("\t",_A(39),"||||",NULL),  //_("edit"
	g_strconcat("\t\t|<Control>j","||",_A(1),".",_A(2),"|ARG",NULL),
	g_strconcat("\t\t|<Control>k","||command|ARG",NULL),
	g_strdup(">"),
	NULL);
}
#endif

  /**********************/
 /**** dialog setup ****/
/**********************/
/**
 * @brief create and show edit dialog
 * @a filename may be full path, or at least the name, of item to be edited.
 * If @a filename is not an absolute path, curr_view->dir is prepended
 * If @a srt is non-NULL, and there's a problem during setup, @a srt is set to NULL
 * @param filename utf8 string, or NULL to edit current output pane buffer
 * @param srt pointer to store for returned dialog data, or NULL
 *
 * @return
**/
static void _e2_edit_dialog_create (gchar *filename, E2_ViewDialogRuntime **srt)
{
	//init view runtime object (0's to ensure view-dialog things are NULL)
	E2_ViewDialogRuntime *rt = ALLOCATE0 (E2_ViewDialogRuntime);
	CHECKALLOCATEDWARN (rt, )
	if (rt == NULL)
	{
		if (srt != NULL)
			*srt = NULL;
		return;
	}
	if (filename != NULL)
	{	//edit file
		rt->textview = gtk_text_view_new ();
		rt->filepath = filename;	//interim store, for the file-read func
		// add the item's content
		if (!e2_view_dialog_read_text (rt))
		{
			gtk_widget_destroy (rt->textview);
			g_free (rt);
			if (srt != NULL)
				*srt = NULL;
			return;
		}
	}
	else
	{	//edit output pane buffer
		const gchar *home = g_get_home_dir ();
		gchar *utf = F_FILENAME_FROM_LOCALE (home);
		gchar *name = g_strconcat (_C(27), "-", _A(39), NULL);	//_("output-edit")
		rt->filepath = g_build_filename (utf, name, NULL);
		rt->charset = "UTF-8";	//the text is from our buffer
		F_FREE (utf);
		g_free (name);
		//textview defaults to editable and cursor visible
		rt->textview = gtk_text_view_new_with_buffer (app.tab.buffer);
		rt->textbuffer = app.tab.buffer;
		GtkTextMark *mark = gtk_text_buffer_get_insert (app.tab.buffer);
		gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (rt->textview),
			mark, 0.2, TRUE, 0.0, 0.5);
	}
#ifdef E2_ASYNC
	//dunno why this gets clobbered when threads used, but ...
	rt->charset = g_strdup (rt->charset);
#endif
	gtk_text_buffer_set_modified (rt->textbuffer, FALSE);
//	rt->is_dirty = FALSE;
	rt->is_hidden = TRUE;
//	rt->undo_list = NULL;
#ifdef E2_REDO_ENABLED
//	rt->redo_list = NULL;
#endif
//	rt->ui_tmp = NULL;
//	rt->seq_reserve = FALSE;	//default value for ui->seq ??

	rt->dialog = e2_dialog_create (NULL, rt->filepath, _("editing file"),
		_e2_edit_dialog_response_cb, rt);
	gtk_window_set_type_hint (GTK_WINDOW (rt->dialog),
		    GDK_WINDOW_TYPE_HINT_NORMAL);
	//override some default label properties
	GtkWidget *label = g_object_get_data (G_OBJECT (rt->dialog),
		"e2-dialog-label");
//	gtk_label_set_line_wrap (GTK_LABEL (label), FALSE);
	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
	//create the view
	GtkWidget *sw = e2_widget_add_sw (GTK_DIALOG (rt->dialog)->vbox,
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, E2_PADDING);
	gtk_container_add (GTK_CONTAINER (sw), rt->textview);
	gtk_widget_show (rt->textview);
	// set view defaults
	rt->textwrap = e2_option_bool_get ("dialog-view-wrap");
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
		rt->textwrap ? GTK_WRAP_WORD : GTK_WRAP_NONE);
	gint char_width, char_height;
	const gchar *fontstr = e2_utils_get_output_font ();
	e2_widget_set_font (rt->textview, fontstr, &char_width, &char_height);
	rt->window_width = e2_option_int_get ("dialog-view-width");
	rt->window_height = e2_option_int_get ("dialog-view-height");;
	rt->idle_id = 0;
	//init search runtime data
	rt->history = e2_list_copy_with_data (find_history);	//CHECKME why copy these ?
	rt->rephistory = e2_list_copy_with_data (replace_history);
	//action area is a GtkHButtonBox, things can't be stacked
	//vertically there
	//so we put the search panel at the bottom of the dialog's vbox
	rt->panel = e2_widget_add_box (GTK_DIALOG (rt->dialog)->vbox,
		FALSE, 0, TRUE, FALSE, 0);
	gtk_widget_hide (rt->panel);
	//add handlebox
	GtkWidget *hndlbox = gtk_handle_box_new ();
	gtk_handle_box_set_shadow_type (GTK_HANDLE_BOX (hndlbox),
		GTK_SHADOW_NONE);
	gtk_handle_box_set_handle_position (GTK_HANDLE_BOX (hndlbox),
		GTK_POS_LEFT);
	gtk_container_add (GTK_CONTAINER (rt->panel), hndlbox);
	gtk_widget_show (hndlbox);
	GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER (hndlbox), vbox);
	//create search bar
	GtkWidget *hbox = e2_view_dialog_create_searchbar (rt);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	//create (hidden) replace bar
	rt->replacebar = hbox = gtk_hbox_new (FALSE, 0);
	//add replace-string combo
	rt->combo2 = e2_combobox_add (hbox,
		FALSE, 0, NULL, rt,
		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
	e2_combobox_append_history (rt->combo2, replace_history);
	e2_view_dialog_update_combo (rt->combo2);
	gtk_widget_set_size_request (rt->combo2, 230, -1);
	GtkWidget *entry = GTK_BIN (rt->combo2)->child;
	g_signal_connect_after (G_OBJECT (entry), "key_release_event",
		G_CALLBACK (_e2_edit_dialog_combokey_cb), rt);
	e2_widget_set_tooltip (NULL, entry, _("Replacements"));
	//add replace-option buttons
	e2_button_add_toggle (hbox, TRUE, do_next,
		_("_repeat"),_("If activated, the next match will be sought after each replacement"),
		do_next, 2, e2_view_dialog_toggled, &do_next);
	e2_button_add_toggle (hbox, TRUE, replace_all,
		_("_all"), _("If activated, all matches will be replaced at once"),
		replace_all, 2, e2_view_dialog_toggled, &replace_all);
	e2_button_add_toggle (hbox, TRUE, confirm_before,
		_("c_onfirm"),_("If activated, confirmation will be sought when \"replacing all\""),
		confirm_before, 2, e2_view_dialog_toggled, &confirm_before);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	//now add things to the action-area
	//produce but don't yet show the "not found" warning
	gchar *labeltext = g_strconcat ("<span weight=\"bold\" foreground=\"",
		e2_option_str_get ("color-negative"), "\">", _("not found"), "</span>", NULL);
	rt->info_label =
	e2_widget_add_label (GTK_DIALOG (rt->dialog)->action_area, labeltext, 0.0, TRUE, 0);
	//left-align the label
	gtk_button_box_set_child_secondary (
		GTK_BUTTON_BOX (GTK_DIALOG (rt->dialog)->action_area), rt->info_label, TRUE);
	gtk_widget_hide (rt->info_label);
	labeltext = _("_Replace");
	rt->replacebtn =
	e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_CONVERT, labeltext, E2_RESPONSE_USER1);
	e2_widget_set_tooltip (NULL, rt->replacebtn, _("Replace this match"));
	gtk_widget_hide (rt->replacebtn);
	replace_keycode = e2_utils_get_mnemonic_keycode (labeltext);
	labeltext = _("_Find");
	rt->findbtn =
	e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_FIND, labeltext, E2_RESPONSE_FIND);
	e2_widget_set_tooltip (NULL, rt->findbtn, _("Find the next match"));
	gtk_widget_hide (rt->findbtn);
	find_keycode = e2_utils_get_mnemonic_keycode (labeltext);
	labeltext = _("_Save");
	rt->savebtn =
	e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_SAVE, labeltext, E2_RESPONSE_USER2);
	//allow saving of a fresh output buffer
	gtk_widget_set_sensitive (rt->savebtn, (filename == NULL));
	save_keycode = e2_utils_get_mnemonic_keycode (labeltext);
	e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_CLOSE);
//	e2_dialog_set_responses (rt->dialog, E2_RESPONSE_FIND, GTK_RESPONSE_CLOSE);
	//make sure we always exit cleanly
	e2_dialog_set_negative_response (rt->dialog, GTK_RESPONSE_CLOSE);
	//some other keycodes are related only to context menu item labels ...
	hide_keycode = e2_utils_get_mnemonic_keycode (_("_Hide"));
	undo_keycode = e2_utils_get_mnemonic_keycode (_("_Undo"));
#ifdef E2_REDO_ENABLED
	redo_keycode = e2_utils_get_mnemonic_keycode (_("Re_do"));
#endif

	//this prevents a check button from being activated by keyboard
//	gtk_dialog_set_default_response (GTK_DIALOG (rt->dialog), E2_RESPONSE_FIND);

	gtk_window_resize (GTK_WINDOW (rt->dialog), char_width * rt->window_width,
				char_height * rt->window_height);
	//setup for cursor management
	GtkSettings *defs = gtk_widget_get_settings (rt->textview);
	if (defs != NULL)
	{
	  g_object_get (defs, "gtk-cursor-blink-time", &blinkmsec, NULL);
	}
	else
		blinkmsec = 1200;	//gtk default
	blinkmsec *= OVERWR_CURSOR_MULTIPLIER;
	rt->blink_init_id = g_idle_add_full (G_PRIORITY_HIGH_IDLE,
		(GSourceFunc)_e2_edit_dialog_blink_start, rt, NULL);

	//initialize undo arrangements
	_e2edtdlg_undo_initialize (rt->textbuffer, rt);

#ifdef E2_TRANSIENTKEYS
	//add dialog-specific key bindings, before the key-press callback
	//group name (must be freeable)
//	rt->key_binding = g_strconcat (_C(16), ".", _A(39), NULL);	//_(general.edit
	rt->key_binding = g_strconcat (_C(9), ".", _A(39), NULL);	//_(dialogs.edit
	e2_keybinding_register_transient (rt->key_binding, GTK_WIDGET (rt->textview),
		_e2_edit_dialog_keybindings);
#endif

	g_signal_connect (G_OBJECT (rt->textbuffer), "modified-changed",
		G_CALLBACK (_e2_edit_dialog_dirty_change_cb), rt);
	g_signal_connect (G_OBJECT (rt->textview), "popup_menu",
		G_CALLBACK (_e2_edit_dialog_popup_menu_cb), rt);
	g_signal_connect (G_OBJECT (rt->textview), "button_press_event",
		G_CALLBACK (_e2_edit_dialog_button_press_cb), rt);
	g_signal_connect (G_OBJECT (rt->textview), "key_press_event",
		G_CALLBACK (e2_edit_dialog_key_press_cb), rt);

	if (srt != NULL)
		*srt = rt;	//make rt available to re-edit func
}

  /******************/
 /***** action *****/
/******************/

/**
 * @brief re-open file for editing, at the last-used location in the file
 * This returns immediately, so if run in a Q-thread, that will end
 * @param from the widget that was activated to initiate the action
 * @param art runtime data for the action
 * @return TRUE if the dialog was opened
 **/
static gboolean _e2_edit_dialog_reopen (gpointer from, E2_ActionRuntime *art)
{
#ifdef E2_ASYNC
	return (e2_task_do_task (E2_TASK_EDIT, art, from,
		_e2_edit_dialog_reeditQ, NULL));
}
static gboolean _e2_edit_dialog_reeditQ (E2_ActionTaskData *qed)
{
#endif
//FIXME allow specification of a name
	//process the 1st selected item in active pane
	#ifdef E2_ASYNC
	GPtrArray *names = qed->names;
#else
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL)
		return FALSE;	//nothing selected
#endif
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	//".." entries filtered when names compiled
	//for name comparisons, we need the full path
#ifdef E2_ASYNC
	gchar *local = e2_utils_strcat (qed->currdir, (*iterator)->filename);
	gchar *edit_this = D_FILENAME_FROM_LOCALE (local);
	g_free (local);
#else
	gchar *utf = F_FILENAME_FROM_LOCALE ((*iterator)->filename);
	//curr_view->dir is utf-8
	gchar *edit_this = g_strconcat (curr_view->dir, utf, NULL);
	F_FREE (utf);
	e2_fileview_clean_selected (names);
#endif
	E2_ViewDialogRuntime *vrt = NULL;
#ifdef E2_ASYNC
	gdk_threads_enter ();
#endif
	_e2_edit_dialog_create (edit_this, &vrt);
#ifdef E2_ASYNC
	gdk_threads_leave ();
#endif
	if (vrt != NULL)
	{
#ifdef E2_ASYNC
		gdk_threads_enter ();
#endif
		e2_view_dialog_show_atlast (edit_this, vrt);
#ifdef E2_ASYNC
		gdk_threads_leave ();
#endif
		g_free (edit_this);
		return TRUE;
	}
	g_free (edit_this);
	return FALSE;
}

  /******************/
 /***** public *****/
/******************/

/**
 * @brief create and show edit dialog
 * @a filename may not have any path, or an absolute path.
 * In those cases, curr_view->dir will be prepended
 * This returns immediately, so if run in a Q-thread, that will end
 * @param filename NULL to edit the output pane, or
 * utf8 string which has at least the name of item to be processed
 * @return TRUE if the dialog was created
 **/
gboolean e2_edit_dialog_create (gchar *filename)
{
	//there is no async thread protection here, as this func can be called from
	//different contexts - any protection is done by the caller
	E2_ViewDialogRuntime *srt;
	_e2_edit_dialog_create (filename, &srt);
	if (srt != NULL)
	{
		e2_dialog_show (srt->dialog, app.main_window, E2_DIALOG_DONT_SHOW_ALL, NULL);
		gtk_window_present (GTK_WINDOW (srt->dialog));
		return TRUE;
	}
	return FALSE;
}
/**
 * @brief register action related to edit dialog
 * @return
 **/
void e2_edit_dialog_actions_register (void)
{
	gchar *action_name = g_strconcat(_A(5),".",_A(40),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_edit_dialog_reopen, NULL, FALSE);
}
/**
 * @brief register config option related to edit dialog
 * @return
 **/
void e2_edit_dialog_options_register (void)
{
	gchar *group_name = g_strconcat(_C(9),":",_A(39),NULL); //_("dialogs:edit"
	//first some options that may, but probably won't, change during the session
	e2_option_bool_register ("edit-save-backup", group_name, _("backup when saving"),
		_("When saving an edited file, an existing file with the same name will be renamed"),
		NULL, FALSE,
		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREEGROUP);
}
